@@ -6,7 +6,7 @@ Convos - Multiuser IRC proxy with web interface
 
 =head1 VERSION
 
-0.2001
+0.3001
 
 =head1 DESCRIPTION
 
@@ -68,28 +68,6 @@ running any of the commands above. Example:
 You can use L<https://github.com/Nordaaker/convos/blob/release/convos.conf>
 as config file template.
 
-=head2 Wanted features
-
-=over 4
-
-=item * Per client state (track seen messages).
-
-=item * Web Notifications that integrate with notification center.
-
-=item * Fast JS Web Interface with async communication (Web Sockets)
-
-=item * Use HTML5 pushstate to be restful and fall back to page reloads for fully functioning non-async lite version.
-
-=item * Monospaced to be compatible with old school IRC clients/ascii
-
-=item * Rich media preview for links.
-
-=item * Facebook Connect for registration/Avatars.
-
-=item * Useful Archive search/viewer
-
-=back
-
 =head2 Architecture principles
 
 =over 4
@@ -102,6 +80,21 @@ as config file template.
 
 =item * Bootstrap-based user interface
 
+=back
+
+=head1 RESOURCES
+
+=over 4
+
+=item * Homepage: L<http://convos.by>
+
+=item * Project page: L<https://github.com/Nordaaker/convos>
+
+=item * Icon: L<https://raw.github.com/Nordaaker/convos/master/public/image/icon.svg>
+
+=item * Logo: L<https://raw.github.com/Nordaaker/convos/master/public/image/logo.svg>
+
+
 =back
 
 =head1 SEE ALSO
@@ -135,7 +128,7 @@ use File::Basename qw( dirname );
 use Convos::Core;
 use Convos::Core::Util ();
 
-our $VERSION = '0.2001';
+our $VERSION = '0.3001';
 $ENV{CONVOS_BACKEND_REV} ||= 0;
 
 =head1 ATTRIBUTES
@@ -144,10 +137,19 @@ $ENV{CONVOS_BACKEND_REV} ||= 0;
 
 Holds a L<Convos::Core::Archive> object.
 
+=head2 cache
+
+Holds a L<Mojolicious::Static> object pointing to a cache dir.
+The directory is "/tmp/convos" by default.
+
 =head2 core
 
 Holds a L<Convos::Core> object.
 
+=head2 backend_pid
+
+The pid for the backend process, if running embedded.
+
 =cut
 
 has archive => sub {
@@ -155,6 +157,16 @@ has archive => sub {
   Convos::Core::Archive->new($self->config->{archive} || $self->path_to('archive'));
 };
 
+has cache => sub {
+  my $self = shift;
+  my $dir = $self->config->{cache_dir} ||= catfile(tmpdir, 'convos');
+
+  $self->log->info("Cache dir: $dir");
+  mkdir $dir or die "mkdir $dir: $!" unless -d $dir;
+
+  Mojolicious::Static->new(paths => [$dir]);
+};
+
 has core => sub {
   my $self = shift;
   my $core = Convos::Core->new;
@@ -163,6 +175,8 @@ has core => sub {
   $core;
 };
 
+has 'backend_pid';
+
 =head1 METHODS
 
 =head2 startup
@@ -189,6 +203,7 @@ sub startup {
   $config->{default_connection}{channels} = [ split /[\s,]/, $config->{default_connection}{channels} ] unless ref $config->{default_connection}{channels};
   $config->{default_connection}{server} = $config->{default_connection}{host} unless $config->{default_connection}{server}; # back compat
 
+  $self->cache; # make sure cache is ok
   $self->plugin('Convos::Plugin::Helpers');
   $self->secret($config->{secret} || die '"secret" is required in config file');
   $self->sessions->default_expiration(86400 * 30);
@@ -213,6 +228,7 @@ sub startup {
   # Normal route to controller
   my $r = $self->routes;
   $r->get('/')->to('client#route')->name('index');
+  $r->get('/avatar/*id')->to('user#avatar')->name('avatar');
   $r->get('/login')->to('user#login', body_class => 'tactile')->name('login');
   $r->post('/login')->to('user#login', body_class => 'tactile');
   $r->get('/register/:invite', { invite => '' })->to('user#register', body_class => 'tactile')->name('register');;
@@ -228,6 +244,8 @@ sub startup {
   $private_r->get('/notifications')->to('client#notification_list', layout => undef)->name('notification_list');
   $private_r->get('/command-history')->to('client#command_history');
   $private_r->post('/notifications/clear')->to('client#clear_notifications', layout => undef)->name('clear_notifications');
+  $private_r->get('/convos')->to('client#convos')->name('convos');
+  $private_r->get('/wizard')->to('user#wizard', body_class => 'tactile')->name('wizard');
   $private_r->get('/settings')->to('user#settings')->name('settings');
   $private_r->post('/settings/connection')->to('user#add_connection')->name('connection.add');
   $private_r->post('/settings/profile')->to('user#edit_user')->name('user.edit');
@@ -238,9 +256,16 @@ sub startup {
   $host_r->get('/')->to('client#view')->name('view.server');
 
   if($config->{backend}{embedded}) {
-    Mojo::IOLoop->timer(0, sub {
-      $self->core->start;
-    });
+    die "Can't run embedded, fork failed: $!" unless defined(my $pid = fork);
+    return $self->backend_pid($pid) if $pid;
+    my $loop=Mojo::IOLoop->singleton;
+    $self->core->start;
+    $0='convos backend';
+
+    $SIG{$_}= 'DEFAULT' for  qw(INT TERM CHLD TTIN TTOU);
+    $SIG{QUIT} = sub{ $loop->max_connnections(0)};
+    $loop->start;
+    exit 0;
   }
 }
 
